;; FUNCTION:
;; ---------
;; DO MIDI *NOTE ON* PROCESSING @ TONE A 
;;
;; 
;; DETAILS:
;; --------
;; THERE HAD BEEN A *NOTE ON* MIDI EVENT
;; JUMP IN HERE FROM ay_chip_note_on_to_hz_tab.inc
;; THE CURRENT MIDI NOTE ON TUNE WAS WRITTEN TO
;; * VAR_MIDI_TO_HZ_LOW_BYTE  (FINE TUNE VALUE)
;; * VAR_MIDI_TO_HZ_HIGH_BYTE (COARSE TUNE VALUE)
;;
;; 
;; MIDI NOTE ON PIPELINE WORKFLOW:
;; -------------------------------
;;
;; * Check MIDI CH 
;; 
;; * CHECK / DO FX PROCESSING
;;
;; * CHECK / DO ENVELOPE HANDLING
;;
;; * CHECK / DO DETUNE (PITCH BEND) HANDLING
;;
;; * SEND TUNE TONE A (HIGH BYTE & LOW BYTE) VALUE to AY (YM) CHIP 
;;
;; * CHECK / DO TONE A VELOCITY / VOLUME HANDLING 
;;
;;
;; 	TONE A, B, C WORKFLOW
;; 	-------------------------------------
;; 	midi_evnt_note_on_pipeline_tone_a.inc
;; 			||
;;			\/
;;	midi_evnt_note_on_pipeline_tone_b.inc
;; 			||
;;			\/
;;	midi_evnt_note_on_pipeline_tone_c.inc
;;			||
;;			\/
;;	note_on_write_ay_reg7.inc
;;

MIDI_EVNT_NOTE_ON_CHECK_MIDI_CHANNEL_TONE_A_B_C

	;; ---------[START TONE A WORKFLOW]------------------------------------------------------
 	;; ----[CHECK: MIDI EVENT NOTE ON = MIDI CH A]-------------------------------------------
 	movf 	MIDI_CURRENT_EVNT_CHANNEL, W
 	cpfseq 	MIDI_CHANNEL_TONE_A_USER
			
			;; ---[STATE: NO, NOT MIDI CH USER TONE A    ]-----------------------------------
			;; ---[DO: GO AND CHECK MIDI CH TONE B       ]-----------------------------------
			goto 	MIDI_EVNT_NOTE_ON_CHECK_MIDI_CHANNEL_B
 		
 		;; YES
 		;; THERE IS A NOTE ON MIDI EVENT @ THE TONE A MIDI CHANNEL
		;; WRITE AY (YM) CHIP MIDI TO Hz VALUES (HIGH BYTE AND LOW BYTE)
		movff 	VAR_MIDI_TO_HZ_HIGH_BYTE,   VAR_TONE_A_TUNE_COARSE_MIDI
		movff 	VAR_MIDI_TO_HZ_LOW_BYTE, 	VAR_TONE_A_TUNE_FINE_MIDI

	;; ---[DO: SET THE MIDI GATE PIN                     ]----------------------------------- 
	bsf AY_MIDI_NOTE_GATE_LAT, AY_MIDI_GATE_PIN_RD4_J14
		
	;; MIDI FILTER BYTE >> MIDI_TONE_ABC_ON_OFF
	;; ---------------------------------------------------------------
	;;
	;; COMMENTARY: AY (YM) CHIP REG7
	;; ----------------[AY CHIP REG 0x07: ASSIGNMENT]-----------------
	;; >>> | BIT7  | BIT6  | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0 |
	;; ---------------------------------------------------------------
	;; >>> | MODE  | MODE  | NOISE| NOISE| NOISE| TONE | TONE | TONE |
	;; >>> | IOB   | IOA   |   C  |   B  |   A  |   C  |   B  |  A   |
	;; >>> | /IN   | /IN   | /ON  | /ON  | /ON  | /ON  | /ON  | /ON  |
	;; >>> | OUT   | OUT   | OFF  | OFF  | OFF  | OFF  | OFF  | OFF  |
	;; ---------------------------------------------------------------
	;; HEX.| 0x80  | 0x40  | 0x20 | 0x10 | 0x08 | 0x04 | 0x02 | 0x01 |
	;; ---------------------------------------------------------------
	;; DEC.| D128  | D064  | D032 | D016 | D008 | D004 | D002 | D001 |
	;; ---------------------------------------------------------------
	;; >>> | ----  | ----  | CLR  | CLR  | CLR  | CLR  |  CLR | CLR  | 	=> TONE ABC AND NOISE ABC = ON
	;; ---------------------------------------------------------------
	;; >>> | ----  | ----  | SET  | SET  | SET  | SET  |  SET | SET  | 	=> TONE ABC AND NOISE ABC = OFF
	;; ---------------------------------------------------------------
	;;
	;;
	;; COMMENTARY: MIDI_TONE_ABC_ON_OFF
	;; ----------------[AY CHIP REG 0x07: ASSIGNMENT]-----------------
	;; >>> | BIT7  | BIT6  | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0 |
	;; ---------------------------------------------------------------
	;; >>> | ----  | ----  | CLR  | CLR  | CLR  | CLR  |  CLR | CLR  | 	=> MIDI FITLER TONE ABC AND NOISE ABC = ON
	;; ---------------------------------------------------------------
	;; >>> | ----  | ----  | SET  | SET  | SET  | SET  |  SET | SET  | 	=> MIDI FILTER TONE ABC AND NOISE ABC = OFF
	;; ---------------------------------------------------------------
	;; 
	;;
	;; NOTICE:
	;; @ AY REG7: BIT CLR = TONE/NOISE ABC ON 
	;; @ AY REG7: BIT SET = TONE/NOISE ABC OFF
	;; MIDI FILTER: SET TONE/NOISE A = ON = CLR MIDI FILTER TONE/NOISE A BIT 
	;; ---[STATE: MIDI EVENT MIDI CHANNEL = MIDI CHANNEL OF TONE A ]------------------		
	;; ---[DO: CLR BIT0_TONE_A_ON_OFF, CLR BIT3_NOISE_A_ON_OFF     ]------------------
	bcf		MIDI_TONE_ABC_ON_OFF, BIT0_TONE_A_ON_OFF
	bcf		MIDI_TONE_ABC_ON_OFF, BIT3_NOISE_A_ON_OFF

		;; SOME MIDI NOTE OFF HANDLING HERE:
		;; STORE THE CURRENT *NOTE ON VALUE* @ MIDI_LAST_NOTE_ON_EVENT_VALUE_TONE_A
		;; FOR CHCKING if a FUTURE MIDI NOTE OFF value CORRELATES WITH THIS
		;; NOTE ON value.
		;; WE NEED THIS FOR A BETTER NOTE OFF HANDLING: 
		;; BECAUSE A FUTURE MIDI NOTE OFF EVENT VALUE 
		;; SHOULD HAVE THE SAME VALUE THAN THE CURRENT MIDI NOTE ON EVENT VALUE 
      	movff	MIDI_EVNT1, MIDI_LAST_NOTE_ON_EVENT_VALUE_TONE_A
	
	;; ------------------------------------------------------------------------------------
	;; ---[FX HANDLING START:                          ]-----------------------------------
	;; ------------------------------------------------------------------------------------
	;; ---[CHECK: FX MODE @ TONE A = ON or OFF ?       ]-----------------------------------
	movff	BNKD_VAR_FX_USER_ON_OFF_ABC, VAR_TMP_RAM_BYTE_COPY_CLIPBOARD
	btfss 	VAR_TMP_RAM_BYTE_COPY_CLIPBOARD, BIT0_FX_ON_OFF_TONE_A
		;; NO, 
		;; BIT0 IS NOT SET
		;; ---[STATE: Fx MODE @ TONE A = OFF           ]----------------------------------
		;; WE DO NOT NEED TO DO FX ROUTINES HERE
		;; GO AND CHECK IF ENV = ON
		rgoto  MIDI_EVNT_NOTE_ON_TONE_A_CHCK_ENVELOPE_MODE
	
		;; YES,
		;; ---[STATE: FX TAB ROUTINES = ON @ TONE A    ]-----------------------------------
		;; ---[DO: STORE THE LAST 3 Note On Values     ]-----------------------------------
		;; USED IN => fx_pipeline.inc
		call 	WRITE_FX_TUNE_1_2_3	
		
		;; ---[DO: RENEW FX MIDI NOTE ON SESSION (inc +1) ]--------------------------------
		;; NOTICE:
		;; EVERY NEW MIDI "NOTE ON" EVENT @ TONE ABC IS A *FX TAB SESSION RESET*
		movff	BNKD_VAR_FX_MIDI_NOTE_ON_SESSION, VAR_TMP_RAM_BYTE_COPY_CLIPBOARD
		incf 	VAR_TMP_RAM_BYTE_COPY_CLIPBOARD, F
		movff	VAR_TMP_RAM_BYTE_COPY_CLIPBOARD, BNKD_VAR_FX_MIDI_NOTE_ON_SESSION
		
		;; ---[DO: SET FX PROCESSING MIDI NOTE ON FLAG @ TONE A ]--------------------------
		movff	BNKD_VAR_Fx_MIDI_NOTE_ON_OR_OFF, VAR_TMP_RAM_BYTE_COPY_CLIPBOARD
		bsf 	VAR_TMP_RAM_BYTE_COPY_CLIPBOARD, BIT0_TONE_A_ON_OFF
		movff	VAR_TMP_RAM_BYTE_COPY_CLIPBOARD, BNKD_VAR_Fx_MIDI_NOTE_ON_OR_OFF
		
		;; ---[FX HANDLING END                         ]-----------------------------------
		;; --------------------------------------------------------------------------------

MIDI_EVNT_NOTE_ON_TONE_A_CHCK_ENVELOPE_MODE
		;; ---[CHECK: ENVELOPE @ TONE A = ON?          ]-----------------------------------
		;; VAR_AY_CHIP_REG8_CH_A_AMPLITUDE: BIT 4 SET = ENV ON
		;; VAR_AY_CHIP_REG8_CH_A_AMPLITUDE: BIT 4 CLR = ENV OFF
		btfsc VAR_AY_CHIP_REG8_CH_A_AMPLITUDE, BIT4_ENVELOPE_ON_OFF
        	;; YES,
			;; BIT 4 is SET - ENV. MODE ON
			;; ----[STAE: ENV @ TONE A = ON ]----------------------------------------------
        	;; GOTO WRITE ENV REG AY CHIP
        	rgoto MIDI_EVNT_NOTE_ON_MIDI_CHANNEL_TONE_A_ENV_MODE
			
			;; NO, 
			;; BIT 4 is CLEAR - ENV. MODE OFF
			;; NO NEED FOR WRITING AY REG 13
			;; GO ON AND CHECK TONE A DETUNE
			;; ----[STAE: ENV @ TONE A = OFF ]----------------------------------------------
			rgoto MIDI_EVNT_NOTE_ON_CHECK_DETUNE_A

	
MIDI_EVNT_NOTE_ON_MIDI_CHANNEL_TONE_A_ENV_MODE
		;; -------------[WRITE ENVELOPE MODE ON]------------------------
		;; WRITE ENV. SHAPE & CYCLE
		;; SEND ENV DATA TO AY REG 13 (ENV. SHAPE/Cycle REG)
		movlw CONST_AY_CHIP_ADR_13_ENVELOPE_SHAPE_AND_CYCLE ; REG ADRESS of AY REGISTER
		movwf VAR_AY_CHIP_REG_ADDRESS 		
		;; WRITE ENVELOPE SHAPE CYCLE DATA TO AY CHIP
		movff VAR_AY_CHIP_REG13_ENV_SHAPE_CYCLE, VAR_AY_CHIP_REG_DATA

		;; --------------[SEND DATA TO AY REG 13]-----------------------------------------------------
		;; WRITING 8BIT DATA (VAR_AY_CHIP_REG_ADDRESS, VAR_AY_CHIP_REG_DATA)TO AY REG(X) ROUTINE
		call LABEL_AY_SR_Write	
		;; ==========================================================================================
			

;; -------------[DETUNE A]------------------
MIDI_EVNT_NOTE_ON_CHECK_DETUNE_A


;; -------------[CHECK DETUNE A (PITCH BEND)]------------------------

;; 	* CHECK DETUNE (PITCH BEND)
;; 	* Coarse
;; 	* Fine 

;; -------------[CHECK COARSE DETUNE TONE A]---------------------------------------
MIDI_EVNT_NOTE_ON_CHECK_TONE_A_COARSE_DETUNE
	;; CHECK USER DETUNE COARSE @ TONE A?
	;: USER DETUNE COARSE RANGE:
	;; (-) RANGE = [0 to 6]|OFFSET/ZERO = [7]| (+) RANGE = [8 to 15]
	;;
	;; VAR_TONE_A_TUNE_COARSE_USER | COARSE DETUNE
	;; ...
	;; 3 ========================>| -4
	;; 4 ========================>| -3
	;; 5 ========================>| -2
	;; 6 ========================>| -1
	;; 7 ========================>|  0
	;; 8 ========================>| +1
	;; 9 ========================>| +2
	
	;; IS USER COARSE TUNE = [7] ?
	movlw	OFFSET_COARSE_TUNE_IS_7
	cpfseq 	VAR_TONE_A_TUNE_COARSE_USER	
		;; NO: NOT 7, SO COARSE USER *DETUNE IS GIVEN*
		rgoto MIDI_EVNT_NOTE_ON_TONE_A_CHECK_COARSE_DETUNE_PLUS_OR_MINUS
		;; YES: COARSE DETUNE = 7, SO *NO COARSE DETUNE IS GIVEN*
		;; GO ON WITH DETUNE FINE
		goto 	MIDI_EVNT_NOTE_ON_CHECK_TONE_A_DETUNE_FINE


MIDI_EVNT_NOTE_ON_TONE_A_CHECK_COARSE_DETUNE_PLUS_OR_MINUS
		;; CHECK COARSE DETUNE = (+) or (-)
		;; COARSE TUNE IS <> 7 SO COARSE REG DETUNE
		;; IS GIVEN @ TONE A
		;; RANGE (-) = [0-6] |(No DETUNE) = [7] | (+) = [8-15]
		movlw	PROOF_POSITIVE_OR_NEGATIVE_VIA_0X06
		cpfsgt 	VAR_TONE_A_TUNE_COARSE_USER	
			;; NO 
			;; SO COARSE DETUNE = (-) = RANGE [0 to 6]
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_COARSE_DETUNE_IS_MINUS
			;; YES
			;; SO COARSE DETUNE = (+) = RANGE [8 to 15]
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_COARSE_DETUNE_IS_PLUS
			

MIDI_EVNT_NOTE_ON_TONE_A_COARSE_DETUNE_IS_MINUS
	;; -----[STATE: DETUNE COARSE IS NEGATIVE                   ]----------------------------------
	;; -----[DO: CALCULATE DETUNE COARSE VALUE                  ]----------------------------------
	;; calculate COARSE DETUNE (-) VALUE:
	;; WREG [(0x07 (+1 [PLUS_1_CARRY_BIT_KILLER])] - F [(0x00 to 0x06)| (+1 Carry BIT (of subfwb))] 
	;; WRITE RESULT TO => [WREG (0x07 to 0x01)]
	;; TO GET THE (-) COARSE TUNE RESULT [RANGE: 0x07 to 0x01]
	
	
	;; (1) MOVE OFFSET TO WREG and
	;; (3)Subtract f from WREG 
	;; the CARRY bit will allways be set
	;; because WREG is (allways in this case) > f) 
	;; (2) SO WE NEED TO ADD +1 TO WREG
	;; for [+1 WREG] - [+1 Carry BIT] = 0 (the needed offset)
	movlw	OFFSET_COARSE_TUNE_IS_7         ;; (1)
	addlw 	PLUS_1_CARRY_BIT_KILLER         ;; (2)
	subfwb	VAR_TONE_A_TUNE_COARSE_USER, W  ;; (3) [SUBFWB f,d,a     Subtract f FROM w with Borrow]
	
	;; ADD (*) COARSE MINUS DETUNE [WREG (RANGE 0x01 to 0x07)] 
	;; TO THE COARSE "NOTE ON VALUE" 
	;; DON'T BE CONFUSED (*):
	;; DETUNE NEGATIVE => LOWER  FREQ = HIGHER TUNE REG VALUE
	;; DETUNE ADDITIVE => HIGHER FREQ = LOWER TUNE REG VALUE
	addwf	VAR_TONE_A_TUNE_COARSE_MIDI, F
	
		;; TUNE COARSE TUNE RANGE = BIN 00000000 to 00001111 (DEC. 0 to 15)
		;; TUNE COARSE TUNE > BIN 00001111? 
		movlw	CONST_NIBBLE_B00001111
		cpfsgt 	VAR_TONE_A_TUNE_COARSE_MIDI	
						;; NO
						;; OK, GO ON WITH DETUNE FINE
						goto 	MIDI_EVNT_NOTE_ON_CHECK_TONE_A_DETUNE_FINE

			;; ---[STATE: COASE TUNE REG VALUE IS > 0x0F NO VALID VALUE FOR THE AY CHIP! ]----
			;; ---[DO: SET TUNE COARSE REG TO 0x0F = MAX VALUE AY CHIP COARSE TUNE REG   ]----
			;; GO AND WRITE MAXIMUM VALUE TO COARSE and FINE REG
			;; WRITE MAXIMUM VALUE TO COARSE and FINE REG
			movlw CONST_COARSE_TUNE_MAX_VALUE_B00001111
			movwf VAR_TONE_A_TUNE_COARSE_MIDI
			movlw CONST_FINE_TUNE_MAX_VALUE_B11010010
			movwf VAR_TONE_A_TUNE_FINE_MIDI

	;; GO ON WITH DETUNE FINE
	rgoto 	MIDI_EVNT_NOTE_ON_CHECK_TONE_A_DETUNE_FINE

;; ------------------------[DETUNE COARSE POSITIVE]----------------------------

MIDI_EVNT_NOTE_ON_TONE_A_COARSE_DETUNE_IS_PLUS
	;; DETUNE IS POSITIVE
	movlw	OFFSET_COARSE_TUNE_IS_7
	subwf	VAR_TONE_A_TUNE_COARSE_USER, W		; - 0x07 (OFFSET IN W REG)
	movwf	VAR_AY_DETUNE_TEMP					; SAVE (+) COARSE DETUNE VALUE

	;; NOW COARSE DETUNE IS IN WREG
	;; (+) COARSE DETUNE RANGE => [0x01 to 0x08]		
	;; COMPARE W REG WITH COARSE TUNE MIDI VALUE
	;; IS THE COASE MIDI VALUE > COASE DETUNE VALUE IN W REG
	;; NEXT STEP: SUBTRACT [(+) DETUNE COARSE] FROM [MIDI COARSE TUNE]
	;; BUT WE MUST FIRS CHECK IF THERE WILL BE A VALID VALUE
	;; COMPARE W REG [(+) DETUNE COARSE] WITH [COARSE TUNE MIDI] 
	;; IS (F) [MIDI COARSE TUNE] > (W) [(+) DETUNE COARSE]?
	cpfsgt 	VAR_TONE_A_TUNE_COARSE_MIDI	
		;; IS (F) [MIDI COARSE TUNE] > (W) [(+) DETUNE COARSE]?
		;; NO - SO SET TUNE REGS TO MIN. VALUE	= HIGHEST TUNE
		rgoto 	MIDI_EVNT_NOTE_ON_SET_MIDI_TUNE_A_TO_HIGHEST_TUNE

		;; IS (F) [MIDI COARSE TUNE] > (W) [(+) DETUNE COARSE]?
		;; YES - OK GO ON
		rgoto 	MIDI_EVNT_NOTE_ON_TONE_A_COARSE_DETUNE_ADDITIVE_GO_ON


MIDI_EVNT_NOTE_ON_SET_MIDI_TUNE_A_TO_HIGHEST_TUNE
			;; SET TUNE REGS TO HIGHEST FREQUENCY POSSIBLE
			;; THAT MEANS TO SET TUNE REGS TO MIN. VALUE
			;; MIN VALUE = HIGHEST TUNE
			movlw  CONST_COARSE_TUNE_MIN_VALUE_B00000000
			movwf  VAR_TONE_A_TUNE_COARSE_MIDI
			movlw  CONST_FINE_TUNE_MIN_VALUE_B00001010
			movwf  VAR_TONE_A_TUNE_FINE_MIDI
						;; GO ON WITH DETUNE FINE
						rgoto 	MIDI_EVNT_NOTE_ON_CHECK_TONE_A_DETUNE_FINE

MIDI_EVNT_NOTE_ON_TONE_A_COARSE_DETUNE_ADDITIVE_GO_ON
			;; ADD THE (+) USER COARSE DETUNE FREQUENCY VALUE NOW
			;; THIS MEANS TO SUBTRACT COARSE DETUNE VALUE from the COARSE "NOTE ON" MIDI VALUE 
			;; YES THE POSITIVE VALUE WILL BE SUBTRACTED (DON'T BE CONFUSED;)
			;; TUNE FREQUENCY "+" = "-" @ TUNE REGISTER
			movf 	VAR_AY_DETUNE_TEMP, W
			subwf	VAR_TONE_A_TUNE_COARSE_MIDI, F
					
						;; GO ON WITH DETUNE FINE
						rgoto 	MIDI_EVNT_NOTE_ON_CHECK_TONE_A_DETUNE_FINE

;; -------------[END OF DETUNE COASE TONE A ]--------------	

;; -------------[CHECK DETUNE FINE Tone A   ]---------------
MIDI_EVNT_NOTE_ON_CHECK_TONE_A_DETUNE_FINE
	;; ------------[CHECK: USER DETUNE FINE (PITCH BEND)] -----------------------------
	movlw	OFFSET_FINE_TUNE_IS_128
	;; TONE A FINE TUNE = 128? 
	;; 128 = OFFSET = DETUNE TONE A FINE = OFF
	cpfseq 	VAR_TONE_A_TUNE_FINE_USER 	
			;; ---[STATE: NO, NOT 128 - SO *DETUNE IS GIVEN*]--------------------------
			;; ---[DO: GO AND CHECK IF DETUNE = [+] or [-]
			rgoto MIDI_EVNT_NOTE_ON_CHECK_DETUNE_A_FINE_IS_PLUS_OR_MINUS

			;; ---[STATE: YES VAR_TONE_A_TUNE_FINE_USER = 128 *NO DETUNE IS GIVEN*]----
			;; ---[DO: GO AND WRITE TUNE TO AY CHIP
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP


MIDI_EVNT_NOTE_ON_CHECK_DETUNE_A_FINE_IS_PLUS_OR_MINUS
	;; CHECK IF USER DETUNE IS (+) OR (-)
	movlw	PROOF_POSITIVE_OR_NEGATIVE_VIA_0X7F
	;; TONE A FINE TUNE > 127? 
	cpfsgt 	VAR_TONE_A_TUNE_FINE_USER	

			;; NO => USER FINE DETUNE => (-)
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_MINUS
			
			;; YES => USER FINE DETUNE => (+)
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_PLUS

;; -----------[PITCH BEND TONE A == NEGATIVE]-----------------------------------------------------------
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_MINUS
	;; DETUNE IS NEGATIVE - LOWER FREQUENCY
	;; (-) RANGE OF: VAR_TONE_A_TUNE_FINE_USER = [0   to 127] DETUNE IS NEGATIVE (HERE)
	;; (NO DETUNE) : VAR_TONE_A_TUNE_FINE_USER = [128       ] NO DETUNE
	;; (+) RANGE OF: VAR_TONE_A_TUNE_FINE_USER = [129 to 255] DETUNE IS POSITIVE
	;; -------------------------------------------------------------------------------------------------
	;; WRITE 128 TO WREG
	;; WREG [(128) + (1 (CARRY KILLER))] - [[VAR_TONE_A_TUNE_FINE_USER |(0 to 127)] + [+1 (CARRY)]]
	;; WRITE RESULT TO => [WREG]
	movlw	OFFSET_FINE_TUNE_IS_128	
	addlw 	PLUS_1_CARRY_BIT_KILLER
	subfwb	VAR_TONE_A_TUNE_FINE_USER, W 
	;; WRITE RESULT TO [WREG] 
	;; STORE W IN VAR_AY_DETUNE_TEMP
	movwf	VAR_AY_DETUNE_TEMP
	;; [VAR_AY_DETUNE_TEMP] RANGE => [1 to 128]
	;; 
	;; So the [USER DETUNE RANGE] => [-1 to -128] 
	;; Expand the (-) RANGE to 8BIT (255):
	;; 2x (-1 to -128) => (-2 to -256(!))
	addwf	VAR_AY_DETUNE_TEMP, F
	;; -1 | => (1 to 255)
	decf	VAR_AY_DETUNE_TEMP, F
	;; WRITE [(-) DETUNE FINE] TO [VAR_AY_DETUNE_TEMP]
	;; NOW THE (-) RANGE = [1 to 255]
	;; 
	;; ------[CHECK: DETUNE NEGATIVE RATIO CHANGE?]-------------------------------------------------------------------------
	;; ------[CHECK WITH: MIDI TUNE COARSE REG > 3]-------------------------------------------------------------------------
	;; CHECK IF WE NEED TO CHANGE THE PITCH BEND RATIO
	;; CHECK IF MIDI TUNE COARSE REG > 3 
	;; 4 to 15 => NO RATIO CHANGE 
	movlw	0x03	 
	cpfsgt 	VAR_TONE_A_TUNE_COARSE_MIDI	

			;; NO [MIDI TUNE COARSE REG = 0 to 3]
			;; THERE MUST BE A PITCH BEND RATIO CONVERSATION
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE
			
			;; YES [MIDI TUNE COARSE REG > 3]
			;; THERE MUST BE *NO* PITCH BEND RATIO CHANGE
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_GO_ON	

MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE
	;; 0) PITCH BEND /16 ==> IF MIDI TUNE COARSE REG == 0
	;; 1) PITCH BEND / 8 ==> IF MIDI TUNE COARSE REG == 1
	;; 2) PITCH BEND / 4 ==> IF MIDI TUNE COARSE REG == 2
	;; 3) PITCH BEND / 2 ==> IF MIDI TUNE COARSE REG == 3
	movf VAR_TONE_A_TUNE_COARSE_MIDI, W
	JUMPTABLE_2BYTES 4
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_16x 	;; 0 => PITCH BEND VALUE /16
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_8x 	;; 1 => PITCH BEND VALUE /8
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_4x 	;; 2 => PITCH BEND VALUE /4
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_2x 	;; 3 => PITCH BEND VALUE /2
	
;; ADJUST PITCH BEND RATIO
;; JUMP IN HERE = PITCH BAND /16
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_16x
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7
;; JUMP IN HERE = PITCH BAND /8
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_8x
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7
;; JUMP IN HERE = PITCH BAND /4
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_4x
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7
;; JUMP IN HERE = PITCH BAND /2
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_RATIO_CHANGE_DIVIDE_2x
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7
	
;; ------[DETUNE (PITCH BEND) RATIO CHANGE END]------------------------------------------------------------------------

MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_NEGATIVE_GO_ON	
	;; >> ADD (-) DETUNE to "NOTE ON" VALUE <<
	;; DON'T BE CONFUSED: 
	;; DETUNE NEGATIVE => LOWER  FREQ = HIGHER TUNE REG VALUE
	;; DETUNE ADDITIVE => HIGHER FREQ = LOWER TUNE REG VALUE
	clrc	;; CLEAR CARRY BIT
	movf	VAR_AY_DETUNE_TEMP, W
	addwf 	VAR_TONE_A_TUNE_FINE_MIDI, F
	    
	;; *CARRY FLAG* is set when an instruction overflows a register.
	;; branch if a carry flag is SET - INC the 4 BIT COARSE TUNE REG
 	bc MIDI_EVNT_NOTE_ON_FINE_DETUNE_NEGATIVE_REG_OVERFLOW_COARSE_REG_A_INC
	
	;; Go AND WRITE TUNE TO AY CHIP
	rgoto MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP

MIDI_EVNT_NOTE_ON_FINE_DETUNE_NEGATIVE_REG_OVERFLOW_COARSE_REG_A_INC
	;; ----[STATE: THE CARRY FLAG HAS BEEN SET (OVERFLOW OF FINE TUNE REG)  ]-------------------------------------------
	;; ----[CHECK: COARSE TUNE REG > BIN xxxx1110? (DEC 14)                 ]------------------------------------------- 
	;; ----[MEMO:  COARSE TUNE REG = 15 = MAX VALUE                         ]-------------------------------------------
	movlw	CONST_COARSE_TUNE_MAX_MINUS_1_B00001110
	cpfsgt 	VAR_TONE_A_TUNE_COARSE_MIDI	

			;; NO - COARSE REG TUNE RANGE IS [0 to 14]
			;; OK SO COASE REG IS VALID
			rgoto MIDI_EVNT_NOTE_ON_CARRY_COARSE_REG_A_INC

			;; YES - NO INC OF COARSE REG POSSIBLE
			;; WRITE MAXIMUM VALUE TO COARSE and FINE REG. 
			rgoto MIDI_EVNT_NOTE_ON_WRITE_MAXIMUM_VALUE_TO_FINE_AND_COARSE_REG_A


MIDI_EVNT_NOTE_ON_CARRY_COARSE_REG_A_INC	
	;; INC PITCH CARRY FROM FINE-TUNE REG 
	;; +1 to COASE TUNE REG 
	incf VAR_TONE_A_TUNE_COARSE_MIDI, F
	
		  		;; Go AND WRITE TUNE TO AY CHIP
		  		rgoto MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP

MIDI_EVNT_NOTE_ON_WRITE_MAXIMUM_VALUE_TO_FINE_AND_COARSE_REG_A
	;; WRITE MAXIMUM VALUE TO COARSE and FINE REG
	movlw  CONST_COARSE_TUNE_MAX_VALUE_B00001111
	movwf  VAR_TONE_A_TUNE_COARSE_MIDI
	movlw  CONST_FINE_TUNE_MAX_VALUE_B11010010
	movwf  VAR_TONE_A_TUNE_FINE_MIDI
	
		 		;; Go AND WRITE TUNE TO AY CHIP
		  		rgoto MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP

; 			
		
;; -----------[PITCH BEND FINE TONE A [+] ]--------------------------------------------
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_PLUS
	;; DETUNE IS POSITIVE - HIGHER FREQUENCY
	;; (+) RANGE OF: VAR_TONE_A_TUNE_FINE_USER = [129 to 255] DETUNE IS POSITIVE (HERE)
	;; (-) RANGE OF: VAR_TONE_A_TUNE_FINE_USER = [0   to 127] DETUNE IS NEGATIVE 
	;; (NO DETUNE) : VAR_TONE_A_TUNE_FINE_USER = [128]
	
	;; OFFSET JUSTAGE
	;; WRITE 128 TO WREG
	movlw	OFFSET_FINE_TUNE_IS_128
	;; [VAR_TONE_A_TUNE_FINE_USER | (129 to 255)] - [WREG|(128)]} WRITE TO => [WREG]
	subwf	VAR_TONE_A_TUNE_FINE_USER, W		; - 0x80 = OFFSET
		;; RANGE JUSTAGE
		;; WRITE [WREG] TO => [VAR_AY_DETUNE_TEMP]
		;; RANGE = [+1 to +128]
		movwf	VAR_AY_DETUNE_TEMP
		;; 2x => (+2 to +256)
		addwf	VAR_AY_DETUNE_TEMP, F
		;; -1 => (+1 to +255)
		decf	VAR_AY_DETUNE_TEMP, F

;; ------[DETUNE RATIO CHANGE?]------------------
	;; CHECK IF WE NEED TO CHANGE THE PITCH BEND RATIO
	;; CHECK IF MIDI TUNE COARSE REG > 3 
	;; 4 to 15 => NO RATIO CHANGE
	movlw	0x03	 
	cpfsgt 	VAR_TONE_A_TUNE_COARSE_MIDI	

			;; NO [MIDI TUNE COARSE REG = 0 to 3]
			;; THERE MUST BE A PITCH BEND RATIO CONVERSATION
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE
			
			;; YES [MIDI TUNE COARSE REG > 3]
			;; THERE MUST BE *NO* PITCH BEND RATIO CHANGE
			rgoto MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_GO_ON

MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE	
	;; 0) PITCH BEND /16 ==> IF MIDI TUNE COARSE REG == 0
	;; 1) PITCH BEND / 8 ==> IF MIDI TUNE COARSE REG == 1
	;; 2) PITCH BEND / 4 ==> IF MIDI TUNE COARSE REG == 2
	;; 3) PITCH BEND / 2 ==> IF MIDI TUNE COARSE REG == 3
	movf VAR_TONE_A_TUNE_COARSE_MIDI, W
	JUMPTABLE_2BYTES 4
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_16x 	;; 0  ==> PITCH BEND VALUE /16
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_8x 	;; 1  ==> PITCH BEND VALUE /8
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_4x 	;; 2  ==> PITCH BEND VALUE /4
		rgoto        MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_2x 	;; 3  ==> PITCH BEND VALUE /2
	
	
	
;; ADJUST PITCH BEND RATIO
;; JUMP IN HERE = PITCH BAND /16
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_16x 
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7
;; JUMP IN HERE = PITCH BAND /8
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_8x
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7
;; JUMP IN HERE = PITCH BAND /4
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_4x
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7
;; JUMP IN HERE = PITCH BAND /2
MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_RATIO_CHANGE_DIVIDE_2x
		rrncf	VAR_AY_DETUNE_TEMP, F
		bcf	VAR_AY_DETUNE_TEMP, 7

;; ------[DETUNE RATIO CHANGE END]------------------

MIDI_EVNT_NOTE_ON_TONE_A_FINE_DETUNE_POSITIVE_GO_ON
;; ------WRITE DETUNE TO MIDI TUNE FINE REG
	
		;; CHECK: DO WE NEED TO SUBTRACT THE NEGATIVE CARRY BIT (BORROW BIT)
		;; FROM THE MIDI TUNE COARSE REG?
		;; IS [DETUNE FINE] > W [TONE FINE MIDI] ?
		;; // [DETUNE FINE] MUST BE = or < THEN [TONE FINE MIDI]
		;; // otherwise WE HAVE DO DEC -1 @ [COARSE MIDI REG]
		movf 	VAR_TONE_A_TUNE_FINE_MIDI, W 
		cpfsgt	VAR_AY_DETUNE_TEMP		 	
						
			;; NO
			;; W [MIDI FINE REG] WAS > F [DETUNE FINE]
			;; or W [MIDI FINE REG]  = F [DETUNE FINE]
			;; SO WE CAN GO ON 
			rgoto MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_WRITE_DETUNE_TO_MIDI_VALUE_TONE_A
		
			;; YES 
			;; F [PITCH BEND] WAS > W [MIDI FINE REG]
			;; SO WE HAVE DO DEC -1 [COARSE MIDI REG]
			rgoto MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_COARSE_REG_A_DEC	


MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_WRITE_DETUNE_TO_MIDI_VALUE_TONE_A	
	;; WRITE THE POSITIVE PITCH BEND TO THE "NOTE ON MIDI VALUE" [FINE TUNE + COARSE TUNE REG]
	;; SUBTRACT DETUNE (THE POSITIVE PITCH BEND) from "NOTE ON" MIDI VALUE 
	;; THE POSITIVE VALUE WILL BE SUBTRACTED (DON'T BE CONFUSED;)
	;; TUNE "+" = "-" @ TUNE REG (LOGIC => AY SOUND CHIP)
	movf	VAR_AY_DETUNE_TEMP, W
	subwf	VAR_TONE_A_TUNE_FINE_MIDI, F
							;; OK, FINE DETUNE IS DONE
							;; Go AND WRITE TUNE TO AY CHIP
							rgoto MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP



MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_COARSE_REG_A_DEC
	;; FIRST: WRITE THE POSITIVE PITCH BEND TO THE "NOTE ON MIDI VALUE" [FINE TUNE + COARSE TUNE REG]
	;; SUBTRACT DETUNE (THE POSITIVE PITCH BEND) from "NOTE ON" MIDI VALUE 
	;; THE POSITIVE VALUE WILL BE SUBTRACTED (DON'T BE CONFUSED;)
	;; TUNE "+" = "-" @ TUNE REG (LOGIC => AY SOUND CHIP)
	movf	VAR_AY_DETUNE_TEMP, W
	subwf	VAR_TONE_A_TUNE_FINE_MIDI, F
	
				;; WE HAVE TO DEC THE VALUE OF COARSE REG -1
				;; BUT CHECK MIDI COASE REG = 0x00 ? 
				;; BECAUSE => 0x00 = INVALID VALUE 
				movlw	0x00
				cpfseq 	VAR_TONE_A_TUNE_COARSE_MIDI

					;; NO - OK COARSE REG IS > 0 - SO GO ON WITH DEC COASE TUNE REG | -1
					rgoto MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_CARRYBIT_DEC_COARSE_REG_A_GO_ON

					;; YES - COARSE REG = 0 
					;; WE CAN NOT DEC. COARSE REG (ALLREADY 0x00)
					;; SO WRITE THE highest PITCH BEND DATA as possible TO THE 
					;; FINE AND COARSE REG instead
					rgoto MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_WRITE_HIGHEST_VALUE_PITCH_A


MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_CARRYBIT_DEC_COARSE_REG_A_GO_ON	
	
						;; DEC DETUNE CARRY FROM FINETUNE REG (MINUS CARY) 
						;; -1 to COASE TUNE REG 
						decf VAR_TONE_A_TUNE_COARSE_MIDI, F
	
						  	;; Go AND WRITE TUNE TO AY CHIP
							rgoto MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP


MIDI_EVNT_NOTE_ON_FINE_DETUNE_POSITIVE_WRITE_HIGHEST_VALUE_PITCH_A
	
						;; THE TUNE WOULD BE OUT OF RANGE 
						;; SO SET FINE TUNE REG TO 0x0A (highest value that is VALID)
						;; COARSE TUNE REG IS ALLREADY 0x00
						movlw 	CONST_FINE_TUNE_MIN_VALUE_B00001010
						movwf	VAR_TONE_A_TUNE_FINE_MIDI
	
							;; Go AND WRITE TUNE TO AY CHIP
							rgoto MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP

;; -------------[END OF DETUNE TONE A FINE ]--------------		


;; -------------[SEND TUNE TONE A DATA TO AY CHIP]---------------------------------------------------
MIDI_EVNT_NOTE_ON_TONE_A_SEND_TUNE_TO_AY_CHIP
	;; SEND TUNE A TO AY CHIP
	movlw  CONST_AY_CHIP_ADR_01_TONE_A_FREQUENCY_PERIOD_COARSE
	movwf  VAR_AY_CHIP_REG_ADDRESS 					; ADDRESS FOR => AY_SR_Write
	
	movf  VAR_TONE_A_TUNE_COARSE_MIDI, W			; DATA FOR AY REG X
	movwf VAR_AY_CHIP_REG_DATA 						; FOR => AY_SR_Write (WRITE DATA TO AY REG)
	
		call LABEL_AY_SR_Write						; WRITING 8BIT DATA 
		;; WRITE VAR_AY_CHIP_REG_ADDRESS, VAR_AY_CHIP_REG_DATA TO AY CHIP	

	movlw  CONST_AY_CHIP_ADR_00_TONE_A_FREQUENCY_FINE
	movwf  VAR_AY_CHIP_REG_ADDRESS 					; ADDRESS FOR => AY_SR_Write

	movf  VAR_TONE_A_TUNE_FINE_MIDI, W				; DATA FOR AY REG X
	movwf VAR_AY_CHIP_REG_DATA 						; FOR => AY_SR_Write (WRITE DATA TO AY REG)
	
		call LABEL_AY_SR_Write						; WRITING 8BIT DATA 
		;; (VAR_AY_CHIP_REG_ADDRESS, VAR_AY_CHIP_REG_DATA)TO AY REG(X) ROUTINE

;; -------------------------------[END OF DETUNE TONE A ]--------------------------------------------

;; ----------------------------[CHECK VELOCITY TO NOISE PERIOD @ TONE A]-------------------------

	;; CHECK IF BIT3 is SET
	btfss VAR_VELOCITY_TO_ABC_NOISE_PERIOD_ON_OFF, BIT3_VELOCITY_TO_NOISE_PERIOD_ON_OFF
		;; NO, BIT3 IS NOT SET
		;; THERE IS NO VELOCITY TO NOISE PERIOD 
		;; SO QUIT HERE AND GO ON WITCH CHECKING THE VELOCITY TO VOLUME @ TONE A
		rgoto  MIDI_EVNT_TONE_A_CHECK_VELOCITY
	
		;; YES - BIT3 IS SET - CHECK IF VELOCITY IS @ TONE A
		;; YES - BIT3 IS SET [VELOCITY TO NOISE PERIOD = ON]
	;; CHECK IF VELOCITY IS @ TONE A => BIT5
	btfss VAR_VELOCITY_TO_ABC_NOISE_PERIOD_ON_OFF, BIT5_VELOCITY_A_TO_NOISE_PERIOD_ON_OFF
		;; NO, BIT5 IS NOT SET
		;; THERE IS NO VELOCITY TO NOISE PERIOD @ TONE A 
		;; SO QUIT HERE AND GO ON WITCH CHECKING THE VELOCITY TO VOLUME @ TONE A
		rgoto MIDI_EVNT_TONE_A_CHECK_VELOCITY
	
	;; YES - BIT5 IS SET - WRITE VELOCITY TONE A TO NOISE PERIOD 
	;; CONVERT MIDI DATA 7BIT (0-127) TO NOISE PERIOD 5BIT(0-31)
	;; [NOISE TONE 5BIT] = [VELOCITY MIDI VALUE 7BIT]/4

	;; WRITE MIDI DATA BYTE2 0-127 
	;; TO VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME
	movff	MIDI_EVNT_VALUE, VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD

     ;; convert VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD 8bit to 5bit (0-31 dec.)
     ;; divide ==> [MIDI VELOCITY VALUE]/4
     rrncf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD, F	 ;; /2
     bcf 		VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD, 7
     rrncf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD, F	 ;; /4
     bcf 		VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD, 7

     movlw 0x1F ;; BIN 0001 1111
     andwf VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD, F

     ;; NOW THE VELOCITY IS IN THE 5 LOWER BITS (0-31 dec.)
     ;; VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD => BIN '000V VVVV'
     ;; REVERSE VELOCITY VALUE
     movlw  CONST_VALUE32_FOR_NOISE_PERIOD_REVERSE_VALUE 
     ;; BIN 0010 0000 (32 dec.) - [MIDI VELOCITY VALUE]/4
     subfwb VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD, F

	movlw 0x1F
	cpfsgt VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD

	      ;; NO, OK
	      ;; VELOCITY TO NOISE PERIOD VALUE IS VALID [xxxV VVVV]
	      rgoto MIDI_EVNT_TONE_A_WRITE_VELOCITY_TO_NOISE_PERIOD_NOW
	
    ;; YES - MAKE VAR VALID
	;; MAKE VELOCITY TO NOISE PERIOD VALUE VALID [xxxV VVVV]
	movlw 0x1F 
	movwf VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD

MIDI_EVNT_TONE_A_WRITE_VELOCITY_TO_NOISE_PERIOD_NOW
	;; -------------[SEND VELOCITY TO NOISE PERIOD REG 6]------------------------
	movlw CONST_AY_CHIP_ADR_06_NOISE_PERIOD		; REG ADRESS 	=> AY REGISTER
	movwf VAR_AY_CHIP_REG_ADDRESS 				; ADDRESS FOR 	=> AY_SR_Write

	;; WRITE VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME TO VAR_AY_CHIP_REG_DATA
	movff VAR_MIDI_VELOCITY_TO_AY_CHIP_NOISE_PERIOD, VAR_AY_CHIP_REG_DATA

		;; --------------[SEND DATA TO AY REG X]-----------------------------------------------------
		call LABEL_AY_SR_Write	; WRITING 8BIT DATA (VAR_AY_CHIP_REG_ADDRESS, VAR_AY_CHIP_REG_DATA)TO AY REG(X) ROUTINE
		;; ==========================================================================================

	;; NEXT Check Velocity

;; ----------------------------[VELOCITY TONE A]-------------------------------
;; ----------------------------------------------------------------------------
MIDI_EVNT_TONE_A_CHECK_VELOCITY
	;; ---[CHECK: SEND VELOYTY TO VOLUME TONE A (BIT0)       ]---------------------- 
	btfsc VAR_VELOCITY_TO_ABC_NOISE_PERIOD_ON_OFF, BIT0_VELOCITY_ON_OFF_TONE_A 
		
		;; NO,
		;; BIT0 = SET 
		;; DO WRITE VELOCITY TO VOLUME
		rgoto  MIDI_EVNT_TONE_A_SEND_VELOCITY_TO_VOLUME
		
		
		;; YES
		;; BIT0 = CLR 
		;; DO NOT SEND VELOCITY TO VOLUME TONE A
		;; WRITE USER VOLUME TO TONE A

	;; -------------[SEND REG8 AMPLITUDE TO TONE A - USER VOLUME 0-15]------------------------
	movlw CONST_AY_CHIP_ADR_08_TONE_A_VOLUME_ENV_MODE_ON_OFF  	; REG ADRESS 	=> AY REGISTER
	movwf VAR_AY_CHIP_REG_ADDRESS 								; ADDRESS FOR 	=> AY_SR_Write

	;; WRITE AY_R8_CH_A_AMPLITUDE TO VAR_AY_CHIP_REG_DATA
	movff VAR_AY_CHIP_REG8_CH_A_AMPLITUDE, VAR_AY_CHIP_REG_DATA

	;; --------------[SEND DATA TO AY REG 13]-----------------------------------------------------
	call LABEL_AY_SR_Write	; WRITING 8BIT DATA (VAR_AY_CHIP_REG_ADDRESS, VAR_AY_CHIP_REG_DATA)TO AY REG(X) ROUTINE
	;; ==========================================================================================
		
	;; NEXT TONE B
	goto MIDI_EVNT_NOTE_ON_CHECK_MIDI_CHANNEL_B


;; --------------------------------------------------------------------
MIDI_EVNT_TONE_A_SEND_VELOCITY_TO_VOLUME
	;; CONVERT MIDI VELOCITY VALUE 7BIT (1-127) TO VOLUME TONE A 4BIT(0-15)
	;; [VOLUME TONE 4BIT] = [MIDI DATA 7BIT]/8

	;; WRITE MIDI DATA BYTE2 0-127 
	;; TO VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME
	movff	MIDI_EVNT_VALUE, VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME

	      ;; convert VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME
	      ;; divide ==> (MIDI DATA)/8
	      rrncf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, F	 ;; /2
	      bcf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, 7
	      rrncf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, F	 ;; /4
	      bcf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, 7
	      rrncf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, F	 ;; /8
	      bcf 	VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, 7

	      movlw 0x0F ;; BIN 00001111
	      andwf VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, F

	      ;; NOW THE VELOCITY IS IN THE 4 LOWER BITS (0-15 dec.)
	      ;; VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME => BIN '0000 VVVV'

	      ;; COPY THE LOWER 4 BIT [BIT0-to 3] = (VELOCITY VOLUME (0-15))
	      ;; TO AY AMPLITUDE REG TONE A = REG 8
	      ;; BUT SAVE THE BIT4 = AMPLITUDE MODE!

	      ;; COPY THE ENV. MODE BIT
	      ;; WRITE 0 TO BIT0-3 
	      movlw 0xF0 ;; BIN 1111 0000
	      andwf VAR_AY_CHIP_REG8_CH_A_AMPLITUDE, W

	      iorwf VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, W
	      movwf VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME

	;; -------------[SEND VELOCITY AMPLITUDE TO TONE A]------------------------
	movlw CONST_AY_CHIP_ADR_08_TONE_A_VOLUME_ENV_MODE_ON_OFF  	; REG ADRESS 	=> AY REGISTER
	movwf VAR_AY_CHIP_REG_ADDRESS 								; ADDRESS FOR 	=> AY_SR_Write

	;; WRITE VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME TO VAR_AY_CHIP_REG_DATA
	movff VAR_MIDI_VELOCITY_TO_AY_CHIP_VOLUME, VAR_AY_CHIP_REG_DATA

	;; --------------[SEND DATA TO AY REG X]-----------------------------------------------------
	call LABEL_AY_SR_Write	; WRITING 8BIT DATA (VAR_AY_CHIP_REG_ADDRESS, VAR_AY_CHIP_REG_DATA)TO AY REG(X) ROUTINE
	;; ==========================================================================================


;; -------------[END VELOCITY TONE A]-------------
;; -------------[END TONE A WORKFLOW]-------------


	;; NEXT TONE B
	;; => note_on_pipeline_b.inc
	goto MIDI_EVNT_NOTE_ON_CHECK_MIDI_CHANNEL_B


